/*
 * adm1_ODE_bsm2.c is a C-file S-function for IAWQ AD Model No 1. 
 * In addition to the ADM1, temperature dependency and dummy states are added.
 * Some details are adjusted for BSM2 (pH inhibition, gas flow output etc).
 *
 * Copyright (2006):
 * Dr Christian Rosen, Dr Darko Vrecko and Dr Ulf Jeppsson
 * Dept. Industrial Electrical Engineering and Automation (IEA)
 * Lund University, Sweden
 * http://www.iea.lth.se/ 
 */

#define S_FUNCTION_NAME Sh2solv_bsm2

#include "simstruc.h"
#include <math.h>

#define XINIT   ssGetArg(S,0)
#define PAR  	ssGetArg(S,1)
#define V	    ssGetArg(S,2)


/*
 * mdlInitializeSizes - initialize the sizes array
 */
static void mdlInitializeSizes(SimStruct *S)
{
    ssSetNumContStates(    S, 0);   /* number of continuous states           */
    ssSetNumDiscStates(    S, 1);   /* number of discrete states             */
    ssSetNumInputs(        S, 74);  /* number of inputs                      */
    ssSetNumOutputs(       S, 1);   /* number of outputs                     */
    ssSetDirectFeedThrough(S, 0);   /* direct feedthrough flag               */
    ssSetNumSampleTimes(   S, 1);   /* number of sample times                */
    ssSetNumSFcnParams(    S, 3);   /* number of input arguments             */
    ssSetNumRWork(         S, 0);   /* number of real work vector elements   */
    ssSetNumIWork(         S, 0);   /* number of integer work vector elements*/
    ssSetNumPWork(         S, 0);   /* number of pointer work vector elements*/
}


/*
 * mdlInitializeSampleTimes - initialize the sample times array
 */
static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}


/*
 * mdlInitializeConditions - initialize the states
 */
static void mdlInitializeConditions(double *x0, SimStruct *S)
{

 x0[0] = mxGetPr(XINIT)[0];

}


/*
 * mdlOutputs - compute the outputs
 */
static void mdlOutputs(double *y, double *x, double *u, SimStruct *S, int tid)
{

    y[0] = x[0]; /* state variable is passed on as output variable */   
 
}

/*
 * mdlUpdate - perform action at major integration time step
 */
static void mdlUpdate(double *x, double *u, SimStruct *S, int tid)
   
{ 
     /* Newton-Raphson algorithm */
    
     double eps, f_h2_su, Y_su, f_h2_aa, Y_aa, Y_fa, Y_c4, Y_pro, K_S_IN, k_m_su, K_S_su, pH_UL_aa, pH_LL_aa, k_m_aa;
     double K_S_aa, k_m_fa, K_S_fa, K_Ih2_fa, k_m_c4, K_S_c4, K_Ih2_c4, k_m_pro, K_S_pro, K_Ih2_pro;
     double pH_UL_ac, pH_LL_ac, k_m_h2, K_S_h2, pH_UL_h2, pH_LL_h2, R, T_base, T_op, kLa, K_H_h2, K_H_h2_base, V_liq, pH_op,I_pH_aa;
     double I_pH_h2, I_IN_lim, I_h2_fa, I_h2_c4, I_h2_pro, inhib[6]; 
     double proc5, proc6, proc7, proc8, proc9, proc10, proc12, p_gas_h2, procT8, reac8;
     double pHLim_aa, pHLim_h2, a_aa, a_h2, S_H_ion, n_aa, n_h2;
     double Equ;
     double gradEqu;
     
     double delta;
     double Sh20;
     int j;
    
     double TOL = 1e-12;
     double MaxSteps= 1000;

     eps = 0.000001;
     Sh20 = x[0]; /* Sh2 */
    
     j = 1; 
     delta = 1.0;
     
     while ( (delta > TOL || delta < -TOL) && (j <= MaxSteps) ) {
         
     f_h2_su = mxGetPr(PAR)[18];
     Y_su = mxGetPr(PAR)[27];
     f_h2_aa = mxGetPr(PAR)[28];
     Y_aa = mxGetPr(PAR)[34];
     Y_fa = mxGetPr(PAR)[35];
     Y_c4 = mxGetPr(PAR)[36];
     Y_pro = mxGetPr(PAR)[37];
     K_S_IN = mxGetPr(PAR)[45];
     k_m_su = mxGetPr(PAR)[46];
     K_S_su = mxGetPr(PAR)[47];
     pH_UL_aa = mxGetPr(PAR)[48];
     pH_LL_aa = mxGetPr(PAR)[49];
     k_m_aa = mxGetPr(PAR)[50];
     K_S_aa = mxGetPr(PAR)[51];
     k_m_fa = mxGetPr(PAR)[52];
     K_S_fa = mxGetPr(PAR)[53];
     K_Ih2_fa = mxGetPr(PAR)[54];
     k_m_c4 = mxGetPr(PAR)[55];
     K_S_c4 = mxGetPr(PAR)[56];
     K_Ih2_c4 = mxGetPr(PAR)[57];
     k_m_pro = mxGetPr(PAR)[58];
     K_S_pro = mxGetPr(PAR)[59];
     K_Ih2_pro = mxGetPr(PAR)[60];
     pH_UL_ac = mxGetPr(PAR)[64];
     pH_LL_ac = mxGetPr(PAR)[65];
     k_m_h2 = mxGetPr(PAR)[66];
     K_S_h2 = mxGetPr(PAR)[67];
     pH_UL_h2 = mxGetPr(PAR)[68];
     pH_LL_h2 = mxGetPr(PAR)[69];
     R = mxGetPr(PAR)[77];
     T_base = mxGetPr(PAR)[78]; 
     T_op = mxGetPr(PAR)[79]; 
     kLa = mxGetPr(PAR)[94];
     K_H_h2_base = mxGetPr(PAR)[98];
     V_liq = mxGetPr(V)[0];
  
     K_H_h2 = K_H_h2_base*exp(-4180.0*(1.0/T_base - 1.0/T_op)/(100.0*R));     /* T adjustment for K_H_h2 */
     
     pH_op = u[59];   /* pH */      // pH_op = u[33];   /* pH */
     S_H_ion = u[60]; /* SH+ */     // S_H_ion = u[34]; /* SH+ */
   
   /* STRs function
   if (pH_op < pH_UL_aa)
    I_pH_aa = exp(-3.0*(pH_op-pH_UL_aa)*(pH_op-pH_UL_aa)/((pH_UL_aa-pH_LL_aa)*(pH_UL_aa-pH_LL_aa)));
   else
    I_pH_aa = 1.0;
   if (pH_op < pH_UL_h2)
    I_pH_h2 = exp(-3.0*(pH_op-pH_UL_h2)*(pH_op-pH_UL_h2)/((pH_UL_h2-pH_LL_h2)*(pH_UL_h2-pH_LL_h2)));
   else
    I_pH_h2 = 1.0;
   */
   
/* Hill function on pH inhibition
pHLim_aa = (pH_UL_aa + pH_LL_aa)/2.0;
pHLim_ac = (pH_UL_ac + pH_LL_ac)/2.0;
pHLim_h2 = (pH_UL_h2 + pH_LL_h2)/2.0;
I_pH_aa = pow(pH_op,24)/(pow(pH_op,24)+pow(pHLim_aa ,24));
I_pH_ac = pow(pH_op,45)/(pow(pH_op,45)+pow(pHLim_ac ,45));
I_pH_h2 = pow(pH_op,45)/(pow(pH_op,45)+pow(pHLim_h2 ,45));
*/

/* MNPs function on pH inhibition, ADM1 Workshop, Copenhagen 2005.
a_aa = 25.0/(pH_UL_aa-pH_LL_aa+eps);
a_ac = 25.0/(pH_UL_ac-pH_LL_ac+eps);
a_h2 = 25.0/(pH_UL_h2-pH_LL_h2+eps);

I_pH_aa = 0.5*(1+tanh(a_aa*(pH_op/pHLim_aa - 1.0)));
I_pH_ac = 0.5*(1+tanh(a_ac*(pH_op/pHLim_ac - 1.0)));
I_pH_h2 = 0.5*(1+tanh(a_h2*(pH_op/pHLim_h2 - 1.0)));
*/

/* Hill function on SH+ used within BSM2, ADM1 Workshop, Copenhagen 2005. */
     pHLim_aa = pow(10,(-(pH_UL_aa + pH_LL_aa)/2.0));
     pHLim_h2 = pow(10,(-(pH_UL_h2 + pH_LL_h2)/2.0));
     n_aa=3.0/(pH_UL_aa-pH_LL_aa);
     n_h2=3.0/(pH_UL_h2-pH_LL_h2);
     I_pH_aa = pow(pHLim_aa,n_aa)/(pow(S_H_ion,n_aa)+pow(pHLim_aa ,n_aa));
     I_pH_h2 = pow(pHLim_h2,n_h2)/(pow(S_H_ion,n_h2)+pow(pHLim_h2 ,n_h2));

     I_IN_lim = 1.0/(1.0+K_S_IN/(u[10]));
     I_h2_fa = 1.0/(1.0+x[0]/K_Ih2_fa);
     I_h2_c4 = 1.0/(1.0+x[0]/K_Ih2_c4);
     I_h2_pro = 1.0/(1.0+x[0]/K_Ih2_pro);
          
     inhib[0] = I_pH_aa*I_IN_lim;
     inhib[1] = inhib[0]*I_h2_fa;
     inhib[2] = inhib[0]*I_h2_c4;
     inhib[3] = inhib[0]*I_h2_pro;
     inhib[5] = I_pH_h2*I_IN_lim;
     
     proc5 = k_m_su*u[0]/(K_S_su+u[0])*u[16]*inhib[0];
     proc6 = k_m_aa*u[1]/(K_S_aa+u[1])*u[17]*inhib[0];
     proc7 = k_m_fa*u[2]/(K_S_fa+u[2])*u[18]*inhib[1];
     proc8 = k_m_c4*u[3]/(K_S_c4+u[3])*u[19]*u[3]/(u[3]+u[4]+eps)*inhib[2];
     proc9 = k_m_c4*u[4]/(K_S_c4+u[4])*u[19]*u[4]/(u[3]+u[4]+eps)*inhib[2];
     proc10 = k_m_pro*u[5]/(K_S_pro+u[5])*u[20]*inhib[3];
     
     proc12 = k_m_h2*x[0]/(K_S_h2+x[0])*u[22]*inhib[5];
     
     p_gas_h2 = u[38]*R*T_op/16.0;  // p_gas_h2 = u[43]*R*T_op/16.0;
     procT8 = kLa*(x[0]-16.0*K_H_h2*p_gas_h2);
     
     reac8 = (1.0-Y_su)*f_h2_su*proc5+(1.0-Y_aa)*f_h2_aa*proc6+(1.0-Y_fa)*0.3*proc7+(1.0-Y_c4)*0.15*proc8+(1.0-Y_c4)*0.2*proc9+(1.0-Y_pro)*0.43*proc10-proc12-procT8; 
     
     /* Sh2 equation */ 
     
     Equ = 1/V_liq*u[26]*(u[7]-x[0])+reac8; // Equ = 1/V_liq*u[26]*(u[51]-x[0])+reac8;    
     
     /* Gradient of Sh2 equation */
     
     gradEqu =  -1/V_liq*u[26]
            -3.0/10.0*(1-Y_fa)*k_m_fa*u[2]/(K_S_fa+u[2])*u[18]*I_pH_aa/(1+K_S_IN/(u[10]))/((1+x[0]/K_Ih2_fa)*(1+x[0]/K_Ih2_fa))/K_Ih2_fa
            -3.0/20.0*(1-Y_c4)*k_m_c4*u[3]*u[3]/(K_S_c4+u[3])*u[19]/(u[4]+u[3]+eps)*I_pH_aa/(1+K_S_IN/(u[10]))/((1+x[0]/K_Ih2_c4)*(1+x[0]/K_Ih2_c4))/K_Ih2_c4
            -1.0/5.0*(1-Y_c4)*k_m_c4*u[4]*u[4]/(K_S_c4+u[4])*u[19]/(u[4]+u[3]+eps)*I_pH_aa/(1+K_S_IN/(u[10]))/((1+x[0]/K_Ih2_c4)*(1+x[0]/K_Ih2_c4))/K_Ih2_c4
            -43.0/100.0*(1-Y_pro)*k_m_pro*u[5]/(K_S_pro+u[5])*u[20]*I_pH_aa/(1+K_S_IN/(u[10]))/((1+x[0]/K_Ih2_pro)*(1+x[0]/K_Ih2_pro))/K_Ih2_pro
            -k_m_h2/(K_S_h2+x[0])*u[22]*I_pH_h2/(1+K_S_IN/(u[10]))+k_m_h2*x[0]/((K_S_h2+x[0])*(K_S_h2+x[0]))*u[22]*I_pH_h2/(1+K_S_IN/(u[10]))
            -kLa;  
    
    /*iteration loop*/
   
        delta = Equ;
        x[0] = Sh20-delta/gradEqu;     /* Calculate the new Sh2 */ 
        
        if (x[0] <= 0) {
            x[0] = 1e-12; /* to avoid numerical problems */
        }
        
        Sh20 = x[0];
        ++j;
    } 
}

/*
 * mdlDerivatives - compute the derivatives
 */
static void mdlDerivatives(double *dx, double *x, double *u, SimStruct *S, int tid)
{

}


/*
 * mdlTerminate - called when the simulation is terminated.
 */
static void mdlTerminate(SimStruct *S)
{
}

#ifdef	MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif

